/**
    \file CWait.hpp
    \author Kurt Stege

    Component that implements a blocking wait mechanism.

    Mainly used by Cogen to generate composite components.
    May be used manually for any purpose.

    History:

    (ks) = Kurt Stege, kurt-stege@online.de

    2005-08-24 (ks):    File created.
    2005-08-24 (ks):    Current version.
*/

#ifndef NSigslot_CWait_hpp_INCLUDEGUARD
#define NSigslot_CWait_hpp_INCLUDEGUARD

#include "Sigslot/TSignature.hpp"

namespace NSigslot {

/*!
    CWait is a Sigslot component that implements a blocking wait.
*/
class CWait
{
    public:

    CWait();
    ~CWait();

    /*!
        This slot waits a long time, until the component is notified
        (from another thread) with \c SLOT_wakeup, and then call the
        \c SIGNAL_work.
    */
    void SLOT_work ();

    /*!
        This slot notifies the component to "wake up". A pending call
        of \c SLOT_work will be triggered to call the \c SIGNAL_work
        as soon as possible.

        When no call of \c SLOT_work is pending, the wake up will be
        used for the next call to \c SLOT_work.
    */
    void SLOT_wakeup ();

    /*!
        Each call of \c SLOT_work() will call this signal once.
    */
    NSigslot::TSignature0::CSignal SIGNAL_work;

    /*!
        During shutdown, the component will not call \c SIGNAL_work
        anymore. Instead, each call to \c SLOT_work will return immediately.
        When a call to \c SLOT_work is pending, the thread resumes and
        returns immediately without calling \c SIGNAL_work anymore.
    */
    void INIT_shutdown();

    private:

    bool    m_wakeupIsActive;
    bool    m_shutdownIsActive;
};

} // namespace NSigslot

#endif // NSigslot_CWait_hpp_INCLUDEGUARD

