/*!
    \file TQueue.hpp
    \author Kurt Stege

    Implements a component for a queue of messages.

    History:

    (ks) = Kurt Stege, kurt-stege@online.de

    2005-08-24 (ks):    File created.
    2005-08-24 (ks):    Current version.
*/


#include "Sigslot/TSignature.hpp"


namespace NSigslot {

/*!
    TODO: Documentation for TQueue.
*/

template <typename TemplateParameterMessage, typename TemplateParameterQueueImplementation>
class TQueue
{
    public:

    typedef TemplateParameterMessage tMessage;  //!< The given template parameter type that is stored in the queue.
    typedef TemplateParameterQueueImplementation tQueueImplementation;  //!< The given template parameter type that implements the actual queue.

    typedef NSigslot::TSignature1<tMessage> tSignature; //!< The signature for \c SIGNAL_message and the \c SLOT \c operator().

    explicit TQueue()
    {
    }

    ~TQueue()
    {
    }

    /*!
        This slot \c operator() inserts a message into the queue.
    */
    void operator()(const tMessage &msg)
    {
        bool notify;

        {
            // CLock lock;
            m_queue.push_front(msg);
            notify = ! m_queue.empty();
        }
        if (notify)
        {
            SIGNAL_notify();
        }
    }

    /*!
        This slot checks, if any message is stored in the queue.

        If so, it removes this message from the queue and calls
        SIGNAL_message with is. Then it returns, even if further
        messages are in the queue.

        If not, that is no message in the queue, the slot
        returns immediately.
    */
    void SLOT_read()
    {
        // CLock lock;

        if (m_queue.empty())
        {
            return;
        }
        tMessage msg = m_queue.back();
        m_queue.pop_back();
        bool notify = ! m_queue.empty();

        // lock.unlock();

        SIGNAL_message(msg);
        if (notify)
        {
            SIGNAL_notify();
        }
    }

    /*!
        This signal is fired from the context of the slot \c operator(),
        when a message is read from the queue.
    */
    tSignature::CSignal SIGNAL_message;

    /*!
        This signal is fired whenever the queue is not empty and
        thus slot \c operator() should be called to read a message:

        - When the slot \c SLOT_read() returns and there is still
          another message in the queue, the slot calls this signal
          to notify the user about this fact.

        - When the slot \c operator() is called and puts the first
          message into the queue (so that it changes from empty to
          not empty) it calls this signal.

        - The component \c TQueue might call the slot at further
          reasons, so don't rely on the fact that the queue is not
          empty when this signal gets fired.
    */
    NSigslot::TSignature0::CSignal SIGNAL_notify;
        
    private:

    tQueueImplementation m_queue;
};

} // namespace NSigslot
